{**********************************************************}
{* 1-bit-full-subtractor unit for LogicGates4 application *}
{**********************************************************}

unit subtractor1f;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, FileUtil, Forms, Controls, Graphics, Dialogs, StdCtrls, ExtCtrls, Grids, common;

type
  {****************}
  { TfSubtractor1f }
  {****************}
  TfSubtractor1f = class(TForm)
    Shape1: TShape;
    StaticText1: TStaticText;
    Label1, Label2, Label3, Label4, Label5, Label6, Label7: TLabel;
    laC1, laC2, laC3: TLabel;
    Shape2, Shape3, Shape4, Shape5, Shape6: TShape;
    Shape11, Shape12, Shape13, Shape14, Shape16, Shape17, Shape18, Shape19: TShape;
    Shape20, Shape21, Shape22, Shape23, Shape24, Shape25, Shape26: TShape;
    Shape27, Shape28, Shape29, Shape30, Shape31, Shape32, Shape33, Shape34: TShape;
    Shape35, Shape36, Shape37, Shape38, Shape39, Shape40, Shape42, Shape43: TShape;
    Shape55, Shape56, Shape58, Shape59, Shape60, Shape62, Shape63, Shape64: TShape;
    Shape66, Shape67, Shape68, Shape69, Shape70, Shape71: TShape;
    Shape72, Shape73, Shape74, Shape75, Shape76: TShape;
    Image1, Image2, Image3, Image5, Image6, Image7: TImage;
    imSwitchA, imSwitchB, imSwitchC: TImage;
    shSwitchA1, shSwitchA2, shSwitchB1, shSwitchB2, shSwitchC1, shSwitchC2: TShape;
    shLedA, shLedB, shLedC: TShape;
    shLedC1, shLedC2, shLedC3: TShape;
    shLedSout, shLedCout: TShape;
    Memo1: TMemo;
    StringGrid1: TStringGrid;
    btClose: TButton;
    procedure FormCreate(Sender: TObject);
    procedure FormActivate(Sender: TObject);
    procedure imSwitchAClick(Sender: TObject);
    procedure imSwitchBClick(Sender: TObject);
    procedure imSwitchCClick(Sender: TObject);
    procedure btCloseClick(Sender: TObject);
  private
    bA, bB, bC: Boolean;
    aLeds: TLeds;
    bLeds: TStatus;
  end;

const
  // Switches description: Data = objects positions (transparent image, long bar shape, short bar shape), horiz. closing move, vert. closing move
  DSwitches: array[0..2] of TSwitchData = (
    (Name: 'SwitchA';
     Data: (104, 108, 111, -1, 0)),
    (Name: 'SwitchB';
     Data: (173, 178, 179, -1, 0)),
    (Name: 'SwitchC';
     Data: (13, 33, 16, 1, 0))
  );

var
  fSubtractor1f: TfSubtractor1f;

implementation

{$R *.lfm}

{****************}
{ TfSubtractor1f }
{****************}

{ Application start: Initialisation }

procedure TfSubtractor1f.FormCreate(Sender: TObject);

begin
  SetLength(aLeds, 8); SetLength(bLeds, 8);
  aLeds[0] := shLedA; aLeds[1] := shLedB; aLeds[2] := shLedC;
  aLeds[3] := shLedC1; aLeds[4] := shLedC2; aLeds[5] := shLedC3;
  aLeds[6] := shLedSout; aLeds[7] := shLedCout;
  laC1.Caption := 'C' + SUB_Digits[1]; laC2.Caption := 'C' + SUB_Digits[2]; laC3.Caption := 'C' + SUB_Digits[3];
end;

{ Form show-up: Initialize 1-bit-full-subtractor simulation}

procedure TfSubtractor1f.FormActivate(Sender: TObject);

var
  I: Integer;

begin
  // All inputs at 0
  bA := False; bB := False; bC := False;
  // Set switches to "off" position
  SwitchOnOff(DSwitches, imSwitchA, shSwitchA1, shSwitchA2, bA);
  SwitchOnOff(DSwitches, imSwitchB, shSwitchB1, shSwitchB2, bB);
  SwitchOnOff(DSwitches, imSwitchC, shSwitchC1, shSwitchC2, bC);
  // Turn LEDs "off"
  for I := 0 to Length(bLeds) - 1 do
    bLeds[I] := False;
  LedsOnOff(aLeds, bLeds);
end;

{ Switch "A" / "B" / "C" activated: Update 1-bit-full-subtractor simulation for actual switch positions }

procedure TfSubtractor1f.imSwitchAClick(Sender: TObject);

begin
  // Change input at A from 0 to Vs (and vice-versa)
  bA := not bA;
  // Led "A" status
  bLeds[0] := bA;
  // Set switches position
  SwitchOnOff(DSwitches, imSwitchA, shSwitchA1, shSwitchA2, bA);
  // Do 1-bit full-subtractor calculation
  Subtractor(bA, bB, bC, bLeds[3], bLeds[4], bLeds[5], bLeds[6], bLeds[7]);
  // Turn LEDs on or off
  LedsOnOff(aLeds, bLeds);
end;

procedure TfSubtractor1f.imSwitchBClick(Sender: TObject);

begin
  // Do the same as above for input B
  bB := not bB;
  bLeds[1] := bB;
  SwitchOnOff(DSwitches, imSwitchB, shSwitchB1, shSwitchB2, bB);
  Subtractor(bA, bB, bC, bLeds[3], bLeds[4], bLeds[5], bLeds[6], bLeds[7]);
  LedsOnOff(aLeds, bLeds);
end;

procedure TfSubtractor1f.imSwitchCClick(Sender: TObject);

begin
  // Do the same as above for input C
  bC := not bC;
  bLeds[2] := bC;
  SwitchOnOff(DSwitches, imSwitchC, shSwitchC1, shSwitchC2, bC);
  Subtractor(bA, bB, bC, bLeds[3], bLeds[4], bLeds[5], bLeds[6], bLeds[7]);
  LedsOnOff(aLeds, bLeds);
end;

{ Button "Close": Close the window }

procedure TfSubtractor1f.btCloseClick(Sender: TObject);

begin
  Close;
end;

end.

